/* Demo showing the use of LAPACK library in a C function. 
This solves exactly the same problem as in real_example.c
but uses double complex datatypes instead of double. 

Author: Devendra Ghate */

#include<stdio.h>

/* Include the function definitions. */
#include <mkl_lapack.h>

/* Include the additional datatypes defined by MKL */
#include <mkl_types.h>

int
main(){

  int N, j, i,tmp1;
  int p, q;
  int info, *ipiv;
  MKL_Complex16 **A, *b,*AT;
  char flag[1];

/* Dimension of the system */
  N = 2;
  
/* Memory allocation of various arrays> */ 
  A = (MKL_Complex16 **)malloc(N*sizeof(MKL_Complex16 *));
  for(i=0;i<N;i++){
     A[i] = (MKL_Complex16 *)malloc(N*sizeof(MKL_Complex16));
  }
  b = (MKL_Complex16 *)malloc(N*sizeof(MKL_Complex16));
  ipiv = (int *)malloc(N*sizeof(int));
  AT = (MKL_Complex16 *)malloc(N*N*sizeof(MKL_Complex16));
 
/* Initializing the A and b matrix. 
b is calculated assuming x[0] = 1 and x[1] = 2. */
  A[0][0].real = 3.0; A[0][1].real = 5.0; b[0].real = 13.0; 
  A[1][0].real = 2.0; A[1][1].real = 4.0; b[1].real = 10.0;       
 
/* As FORTRAN stores the arrays along the columns, 
while C stores the arrays along the rows, we need
to transpose the matrix first to pass it to LAPACK.
Also, if the output from a LAPACK subroutine consists
of a matrix then it has to be transposed back before 
using it in C. Note here that AT (Transpose of A) is
created as a single dimensional array as FORTRAN would 
reference it. */
  for (p=0; p<N; p++){
    for(q=0; q<N; q++) AT[q+N*p]=A[q][p];           
  }  

/* FORTRAN passes all the function arguments by reference.
Hence we need to pass the pointers of each variable instead
of the value. */
  zgetrf(&N, &N, AT, &N, ipiv, &info);

/* For passing constants like here, you need to attach a 
variable to it before passing it by reference to any 
LAPACK subroutine. */
  tmp1 = 1;
  sprintf(flag,"n");
  zgetrs(flag, &N, &tmp1, AT, &N, ipiv, b, &N, &info);
 
/* This is the answer (x). */ 
  printf("Solution x is\n");
  for (p=0; p<N; p++){
      printf("%f\n",b[p].real);
  }
  return 0;
}
